# Twitter API Integration for Preside

This extension allows you to configure multiple Twitter accounts in your Preside application, and to interact with the Twitter API.

A major feature is the ability to pull tweet data from Twitter and store it locally.

## Benefits

- Custom rendering of timelines and individual tweets: tweets can be placed and formatted however desired
- Speed of rendering: tweets are rendered as part of the page (and can be cached if desired), with no external requests to Twitter
- Indexing: tweets can be indexed and searched both locally and by search engines
- No tracking: because no 3rd-party widgets are being rendered by your page, no 3rd-party tracking is happening either

## Prerequisites

Because of the nature of Twitter statuses, which will very often include emojis, you **MUST** have your MySQL database set up to use the `utf8mb4` character set.

(This is probably best practice these days anyway, especially if your application is going to be storing _any_ user-supplied data.)

## Getting started

1. Add one (or more) Twitter accounts through **Data Manager > Twitter > Twitter accounts**. You will need to [set up API access at Twitter](https://developer.twitter.com/en/apps/) in order to obtain the required access credentials.
2. You may configure a "start date" for each account - this will be the date from which your tweet history will be imported.
3. In **Task Manager > Twitter**, run the **Get new tweets** task (or wait for it to run on schedule).

The `twitter_status` Preside object will now hold a local history of all your tweets, and new tweets will continually be added to this.

**NOTE:** the `twitter_status` object uses Twitter's own status ID as its ID property

## Scheduled tasks

In addition to the **Get new tweets** task, there are other tasks in the Task Manager:

- **Re-render tweets**: this will re-render each local tweet using the _default_ context (see below for more information) and store the result in the `twitter_status` object
- **Update counts/deletions**: these run at different frequencies to update like/retweet counts, and remove from the local system any tweets that have since been deleted. Out of the box, every 10 minutes it will check tweets that were posted in the past day; every 2 hours it will check tweets from the past 30 days; and once a day it will check _all_ locally-stored tweets.

## Rendering tweets

A number of default renderers are provided which take the raw data from the API and render it into a formatted tweet that can be included on a web page. This includes linking users, hashtags and URLs, and embedding media items.

The renderers provided are:

- `tweet_hashtags`
- `tweet_symbols`
- `tweet_urls`
- `tweet_user_mentions`
- `tweet_media_photo`
- `tweet_media_video`
- `tweet_media_animated_gif`

Finally, there is a `tweet` renderer which combines the text and media into one formatted HTML block.

Each of these renderers has a `default` context, which is what is used by the import task to render tweets for general display. You can override these in your application and style the result as you desire.

By adding alternate contexts for some or all of these renderers, you can create different layouts for your tweets for different display purposes. For example, you might create a `featured` context which renders some aspects of the tweet differently. You could then render a single tweet to your page using the `renderTweet` helper (which is a proxy to the `TweetRenderingService`):

```
	// By passing the raw tweet data returned from the API and stored locally
	tweet     = presideObjectService.selectData( objectName="tweet_status", id=tweetId );
	tweetData = deserializeJson( tweet.raw_data );
	rendered  = renderTweet( tweet=tweetData, context="featured" );
```
or

```
	// By simply supplying a tweet ID
	rendered = renderTweet( tweet=tweetId, context="featured" );
```

**NOTE:** unlike the default rendering that occurs on import, rendering tweets in this way will not persist the rendered result, so you may want to implement your own caching for this.

## On the roadmap

There will be other functionality added in the future, to interact with other aspects of the Twitter API.

One of the key features will be the ability to **post tweets from your application** to any of your configured accounts. For instance, you could automatically post a status update if you add a new article or feature.

There will also be some default stylings to accompany the default tweet rendering.