/**
 * @presideService
 * @singleton
 */
component {
// CONSTRUCTOR
	/**
	 *
	 */
	public any function init() {
		return this;
	}

	public string function renderTweet( required any tweet, string context="default" ) {
		var tweet          = _getTweetData( tweet );

		if ( tweet.isEmpty() ) {
			return "";
		}

		var entities        = _combineAndSortEntities( tweet );
		var mediaEntities   = _combineMediaEntities( tweet );
		var renderedBody    = tweet.full_text ?: "";
		var stringToReplace = "";
		var replaceWith     = "";
		var media           = [];
		var mediaType       = "";
		var renderedEntity  = "";
		var placeholder     = "";
		var placeholders    = {};

		for( var entity in entities ) {
			stringToReplace = _getStringToReplace( entity );
			placeholder     = "";
			if ( entity.type != "media" ) {
				placeholder = "{#createuuid()#}";
				placeholders[ placeholder ] = _renderEntity( entity, arguments.context );
			}

			renderedBody = replacenocase( renderedBody, stringToReplace, placeholder, "one" );
		}
		renderedBody = replace( renderedBody, placeholders );

		for( var mediaEntity in mediaEntities ) {
			mediaType = mediaEntity.data.type;
			media.append( _renderEntity( mediaEntity, arguments.context ) );
		}

		return $renderContent(
			  renderer = "tweet"
			, data     = { tweet=tweet, renderedBody=renderedBody, media=media, mediaType=mediaType }
			, context  = arguments.context
		);
	}

	public string function renderTweetDate( required date date ) {
		var dateMask   = "d mmm";
		var parsedDate = parseDateTime( arguments.date );

		if ( now() - parsedDate < 1 ) {
			return dateDiff( "h", parsedDate, now() ) & "h";
		}
		if ( year( now() ) != year( parsedDate ) ) {
			dateMask = "d mmm, yyyy";
		}

		return dateformat( parsedDate, dateMask );
	}


	private string function _getStringToReplace( required struct entity ) {
		var data = arguments.entity.data;

		switch( arguments.entity.type ) {
			case "hashtags":
				return "##" & data.text;
			case "symbols":
				return "$" & data.text;
			case "user_mentions":
				return "@" & data.screen_name;
			case "urls":
			case "media":
				return data.url;
		}
	}

	private string function _renderEntity( required struct entity, string context="default" ) {
		var type     = arguments.entity.type;
		var renderer = "tweet_#type#";

		if ( type == "media" ) {
			renderer &= "_#arguments.entity.data.type#";
		}

		return trim( $renderContent( renderer=renderer, data=arguments.entity.data, context=arguments.context ) );
	}

	private query function _getTweetById( required string id ) {
		return $getPresideObject( "twitter_status" ).selectData( id=arguments.id );
	}

	private struct function _getTweetData( required any tweet ) {
		if ( isStruct( arguments.tweet ) ) {
			return arguments.tweet;
		}

		if ( isNumeric( arguments.tweet ) ) {
			var tweet = _getTweetById( arguments.tweet );
			if ( tweet.recordcount ) {
				return deserializeJson( tweet.raw_data );
			}
		}

		return {};
	}

	private array function _combineAndSortEntities( required struct tweet ) {
		var entities    = [];
		var allEntities = arguments.tweet.entities ?: {};

		for( var type in allEntities ) {
			if ( type == "polls" ) {
				continue;
			}
			for( var entity in allEntities[ type ] ) {
				entities.append( { type=type, data=entity } );
			}
		}

		entities.sort( function ( entity1, entity2 ) {
			var start1 = entity1.data.indices[ 1 ];
			var start2 = entity2.data.indices[ 1 ];

			if ( start1 > start2 ) {
				return 1;
			} else if ( start1 < start2 ) {
				return -1;
			} else {
				return 0;
			}
		} );

		return entities;
	}

	private array function _combineMediaEntities( required struct tweet ) {
		var entities      = [];
		var mediaEntities = arguments.tweet.extended_entities.media ?: [];
		var site          = $getRequestContext().getSite();
		var useHttps      = ( site.protocol ?: "http" ) == "https";
		var mediaUrlVar   = useHttps ? "media_url_https" : "media_url";
		var itemUrl       = "";
		var itemExtension = "";
		var itemBaseUrl   = "";
		var tweetId       = arguments.tweet.id_str ?: "";

		mediaEntities.each( function( entity, index, array ) {
			itemUrl       = entity[ mediaUrlVar ] ?: "";
			itemExtension = listLast( itemUrl, "." );
			itemBaseUrl   = rereplace( itemUrl, "\.#itemExtension#$", "" );

			entity.itemData = {
				  tweetId   = tweetId
				, count     = array.len()
				, index     = index
				, url       = itemUrl
				, baseUrl   = itemBaseUrl
				, extension = itemExtension
			};

			entities.append( { type="media", data=entity } );
		} );

		return entities;
	}

}